@@ -3,7 +3,7 @@ use warnings;
 no warnings 'uninitialized';
 
 package Form::Diva;
-$Form::Diva::VERSION = '0.06';
+$Form::Diva::VERSION = '0.0650';
 # ABSTRACT: Generate HTML5 form label and input fields
 
 use Storable qw(dclone);
@@ -22,8 +22,10 @@ sub new {
     $self->{class} = $class;
     unless ( $self->{input_class} ) { die 'input_class is required.' }
     unless ( $self->{label_class} ) { die 'label_class is required.' }
+    ( $self->{HiddenMap}, $self->{HiddenHash} )
+        = &_expandshortcuts( $self->{hidden} );  
     ( $self->{FormMap}, $self->{FormHash} )
-        = &_expandshortcuts( $self->{form} );
+        = &_expandshortcuts( $self->{form} );      
     return $self;
 }
 
@@ -33,6 +35,8 @@ sub clone {
     my $new   = {};
     my $class = 'Form::Diva';
     $new->{FormHash}    = dclone $self->{FormHash};
+    $new->{HiddenHash}    = dclone $self->{HiddenHash};
+    $new->{HiddenMap}    = dclone $self->{HiddenMap};
     $new->{input_class} = $args->{input_class} || $self->{input_class};
     $new->{label_class} = $args->{label_class} || $self->{label_class};
     $new->{form_name}   = $args->{form_name} || $self->{form_name};
@@ -160,11 +164,6 @@ sub _input {
         $input = qq|<TEXTAREA $B{name} $B{id}
         $B{input_class} $B{placeholder} $B{extra} >$B{rawvalue}</TEXTAREA>|;
     }
-    #hidden fields don't get a class or a placeholder
-    elsif ( $B{hidden}) {
-        $input .= qq|<INPUT $B{type} $B{name} $B{id}
-        $B{extra} $B{value} >|;        
-    }
     else {
         $input .= qq|<INPUT $B{type} $B{name} $B{id}
         $B{input_class} $B{placeholder} $B{extra} $B{value} >|;
@@ -174,6 +173,19 @@ sub _input {
     return $input;
 }
 
+sub _input_hidden {
+    my $self  = shift;
+    my $field = shift;
+    my $data  = shift;
+    my %B     = $self->_field_bits( $field, $data );
+    #hidden fields don't get a class or a placeholder
+    my $input .= qq|<INPUT type="hidden" $B{name} $B{id}
+        $B{extra} $B{value} >|;
+    $input =~ s/\s+/ /g;     # remove extra whitespace.
+    $input =~ s/\s+>/>/g;    # cleanup space before closing >
+    return $input;
+}
+
 # generates the id= for option items.
 # uses global %id_uq to insure uniqueness in generated ids.
 # It might be cleaner to make this a sub ref under _option_input
@@ -244,7 +256,6 @@ sub generate {
     my $self    = shift;
     my $data    = shift;
     my $overide = shift;
-    unless ( keys %{$data} ) { $data = undef }
     my @generated = ();
     $self->_clear_id_uq ;    # needs to be empty when form generation starts.
     foreach my $field ( @{ $self->{FormMap} } ) {
@@ -273,4 +284,14 @@ sub generate {
     return \@generated;
 }
 
+sub hidden {
+    my $self    = shift;
+    my $data    = shift;
+    my $output = '';
+    foreach my $field ( @{ $self->{HiddenMap} } ) {    
+        $output .= $self->_input_hidden( $field, $data ) . "\n" ;
+        }
+    return $output ;
+}
+
 1;
